Package: CSwR
Title: Companion to the Book "Computational Statistics with R"
Version: 0.1.3
Authors@R: 
    c(
      person(
        "Niels Richard",
        "Hansen",
        email = "Niels.R.Hansen@math.ku.dk",
        role = "aut",
        comment = c(ORCID = "0000-0003-3883-365X")
      ),
      person(
        "Johan",
        "Larsson",
        role = c("aut", "cre"),
        email = "johan@jolars.co",
        comment = c(ORCID = "0000-0002-4029-5945")
      )
    )
Description: Provides data sets and functions used in the book
    "Computational Statistics with R" (<https://cswr.nrhstat.org>).
Depends: R (>= 3.5.0)
Imports: bench, ggplot2, rlang
Suggests: covr, spelling, testthat (>= 2.1.0)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Language: en-US
URL: https://jolars.github.io/CSwR-Package/, https://cswr.nrhstat.org/
BugReports: https://github.com/jolars/CSwR-Package/issues
License: MIT + file LICENSE
NeedsCompilation: no
Packaged: 2026-05-28 07:19:54 UTC; jola
Author: Niels Richard Hansen [aut] (ORCID:
    <https://orcid.org/0000-0003-3883-365X>),
  Johan Larsson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4029-5945>)
Maintainer: Johan Larsson <johan@jolars.co>
Repository: CRAN
Date/Publication: 2026-06-01 08:40:20 UTC
Built: R 4.5.2; ; 2026-06-01 09:42:14 UTC; unix

Package: CSwR
Title: Companion to the Book "Computational Statistics with R"
Version: 0.1.3
Authors@R: 
    c(
      person(
        "Niels Richard",
        "Hansen",
        email = "Niels.R.Hansen@math.ku.dk",
        role = "aut",
        comment = c(ORCID = "0000-0003-3883-365X")
      ),
      person(
        "Johan",
        "Larsson",
        role = c("aut", "cre"),
        email = "johan@jolars.co",
        comment = c(ORCID = "0000-0002-4029-5945")
      )
    )
Description: Provides data sets and functions used in the book
    "Computational Statistics with R" (<https://cswr.nrhstat.org>).
Depends: R (>= 3.5.0)
Imports: bench, ggplot2, rlang
Suggests: covr, spelling, testthat (>= 2.1.0)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Language: en-US
URL: https://jolars.github.io/CSwR-Package/, https://cswr.nrhstat.org/
BugReports: https://github.com/jolars/CSwR-Package/issues
License: MIT + file LICENSE
NeedsCompilation: no
Packaged: 2026-05-28 07:19:54 UTC; jola
Author: Niels Richard Hansen [aut] (ORCID:
    <https://orcid.org/0000-0003-3883-365X>),
  Johan Larsson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4029-5945>)
Maintainer: Johan Larsson <johan@jolars.co>
Repository: CRAN
Date/Publication: 2026-06-01 08:40:20 UTC
Built: R 4.5.2; ; 2026-06-01 09:28:20 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",tracer)
S3method(autoplot,trace)
S3method(autoplot,tracer)
S3method(print,terminator)
S3method(print,tracer)
S3method(summary,tracer)
export(force_all)
export(plotter)
export(rng_stream)
export(terminator)
export(tracer)
import(ggplot2)
importFrom(rlang,.data)

# CSwR

<!-- badges: start -->

[![R build
status](https://github.com/jolars/CSwR-Package/workflows/R-CMD-check/badge.svg)](https://github.com/jolars/CSwR-Package/actions)
[![Code
coverage](https://codecov.io/gh/jolars/CSwR-Package/graph/badge.svg)](https://app.codecov.io/gh/jolars/CSwR-Package)
<!-- badges: end -->

The CSwR package is a companion to the book *Computational Statistics
with R* available at [cswr.nrhstat.org](https://cswr.nrhstat.org). It
provides data and utility functions used throughout the book for
teaching computational statistics concepts.

## Installation

You can install the latest version from CRAN with:

``` r
install.packages("CSwR")
```

Or install the development version from GitHub with:

``` r
# install.packages("pak")
pak::pak("jolars/CSwR-Package")
```

## Features

The package includes:

- **Datasets**: Five datasets used in examples throughout the book:

  - `greenland` - Monthly temperature data from southwest Greenland
    (1873-2013)
  - `nuuk` - Annual temperature summaries for Nuuk (1867-2013)
  - `vegetables` - Weekly frozen vegetable sales data
  - `angle` - Torsion angles from protein structure (1HMP)
  - `news` - Social media news sharing features from Mashable articles

- **Tracer objects** (`tracer()`) - Collect, print, and visualize trace
  information during function evaluation for debugging and performance
  analysis

- **Terminator objects** (`terminator()`) - Define termination
  conditions for iterative algorithms with callback support

- **Random number streams** (`rng_stream()`) - Efficient cached random
  number generation

- **Force evaluation** (`force_all()`) - Utility to force evaluation of
  all function arguments and avoid lazy evaluation pitfalls

- **Plotter expressions** (`plotter()`) - Create expressions for
  iterative plot updates during long-running computations

## Usage

``` r
library(CSwR)

# Load and explore datasets
data(greenland)
head(greenland)
```

      Year Month Temp_nuuk Temp_Qaqortoq Temp_diff
    1 1873     1     -12.1          -9.9      -2.2
    2 1874     1     -13.1         -11.6      -1.5
    3 1875     1      -6.6          -6.3      -0.3
    4 1876     1     -11.1          -8.3      -2.8
    5 1877     1     -14.5          -8.8      -5.7
    6 1878     1      -7.7          -6.8      -0.9

``` r
# Create a tracer for monitoring algorithm progress
tr <- tracer(c("x", "residual"), time = TRUE)
# Use tr$tracer as a callback in your iterative function

# Set up a terminator with a convergence condition
term <- terminator(cond = quote(residual < 1e-6), Delta = 10)
# Returns TRUE when condition is met
```

## Documentation

For detailed documentation and examples, visit the [package
website](https://jolars.github.io/CSwR-Package/) or see the book at
[cswr.nrhstat.org](https://cswr.nrhstat.org).

## License

MIT License

#' Constructor of a tracer object
#'
#' Tracer objects can collect, print and summarize trace information from
#' the evaluation environment of other functions during their evaluation.
#'
#' The function \code{tracer} constructs a tracer object containing a
#' \code{tracer}, a \code{get} and a \code{clear} function. A call of
#' the objects \code{tracer} function can
#' be inserted in other functions and used to collect, print and plot trace information
#' about the internals of that function during its evaluation. The objects \code{get}
#' function can access that information afterwards, and its \code{clear} function deletes
#' all stored values in the tracer object.
#'
#' A tracer object can trace time (in seconds) between \code{tracer} calls,
#' which are measured by
#' the \code{hires_time} function from the bench package. There are \code{print}
#' and \code{summary} methods available for summarizing the trace information.
#' A call of the \code{tracer} function can be manually inserted into the body of
#' the function that is to be traced, it can be inserted using \code{base::trace},
#' or it can be passed as an argument to any function with a callback argument.
#'
#' @param objects a character vector of names of the objects that are to be traced.
#'                The objects are searched for in the calling environment of the
#'                \code{tracer} function. Objects created by the \code{expr} argument
#'                can also be traced.
#' @param Delta   an integer specifying if and how often trace information is printed.
#'                \code{Delta = 0} means never, and otherwise trace information is printed
#'                every \code{Delta}-th iteration. \code{Delta = 1} is the default.
#' @param save    a logical value. Determines if the trace information is stored.
#' @param time    a logical value. Determines if runtime information in seconds
#'                is traced.
#' @param expr    an expression that will be evaluated in an environment that has
#'                the calling environment of the \code{tracer} function as parent.
#' @param plotter an expression, possibly created by the \code{plotter} function.
#' @param ...     other arguments passed to \code{format} for printing and
#'                \code{plot.window}
#'
#' @return A tracer object containing the functions \code{tracer}, \code{get} and
#' \code{clear}.
#' @export
#'
#' @examples
#' test_tracer <- tracer(c("m", "m_sq"), expr = quote(m_sq <- m^2))
#'
#' test <- function(n, cb = NULL) {
#'   for(i in 1:n) {
#'     m <- 2 * i
#'     Sys.sleep(0.1)
#'     if (!is.null(cb)) cb()
#'   }
#' }
#'
#' test(10, test_tracer$tracer)
#' summary(test_tracer)
tracer <- function(
  objects = NULL,
  Delta = 1,
  save = TRUE,
  time = TRUE,
  expr = NULL,
  plotter = NULL,
  ...
) {
  force(objects)
  force(Delta)
  force(time)
  force(save)

  gc_last_time <- gc.time()[3]
  values_save <- list()
  last_time <- bench::hires_time()
  eval_expr <- is.call(expr) || is.expression(expr)
  plotter_expr <- is.call(plotter) || is.expression(plotter)

  # Environment for evaluation of the expression or plotter
  env <- new.env()
  if (plotter_expr) {
    new_plotter_window(...)
  }
  env$.__n <- 1

  tracer <- function() {
    time_diff <- bench::hires_time() - last_time
    n <- env$.__n
    gc_time <- gc.time()[3]
    gc_time_diff <- gc_time - gc_last_time
    if (gc_time_diff > 0) {
      time_diff <- max(time_diff - gc_time_diff, 0)
    }

    # The expression is evaluated in its own environment to ensure that it
    # does not accidentally overwrite variables in the calling environment.
    # To give the expression access to the calling environment it is assigned
    # as the enclosing environment.
    envir <- parent.frame()
    parent.env(env) <- envir

    if (eval_expr) {
      tryCatch(eval(expr, envir = env), error = function(e) warning(e))
    }

    if (is.null(objects)) {
      objects <- ls(envir)
    }

    values <- mget(objects, envir = envir, ifnotfound = list(NA))

    if (eval_expr) {
      na_val <- unlist(lapply(values, function(x) all(is.na(x))))
      values <- values[!na_val]
      expr_objects <- objects[!(objects %in% names(values))]
      values <- c(
        values,
        mget(expr_objects, envir = env, ifnotfound = list(NA))
      )
    }

    if (isTRUE(Delta > 0) && (n == 1 || n %% Delta == 0)) {
      if (plotter_expr) {
        tryCatch(eval(plotter, envir = env), error = function(e) warning(e))
      }
      cat(
        "n = ",
        n,
        ": ",
        paste(names(values), " = ", format(values, ...), "; ", sep = ""),
        "\n",
        sep = ""
      )
    }

    if (save) {
      if (time) {
        values[[".time"]] <- time_diff
      }
      values_save[[n]] <<- values
    }

    env$.__n <- n + 1
    gc_last_time <<- gc.time()[3]
    last_time <<- bench::hires_time()

    invisible(NULL)
  }

  get <- function(simplify = FALSE) {
    if (simplify) {
      col_names <- unique(unlist(lapply(values_save, names)))
      values_save <- lapply(values_save, function(entry) {
        lapply(entry, col_to_row)
      })
      values_save <- lapply(
        col_names,
        function(x) {
          do.call(
            rbind,
            unlist(lapply(values_save, function(y) y[x]), recursive = FALSE)
          )
        }
      )
      names(values_save) <- col_names
      values_save <- lapply(
        col_names,
        function(x) {
          x_val <- values_save[[x]]
          if (!is.null(ncol(x_val)) && ncol(x_val) == 1) {
            colnames(x_val) <- x
          } else {
            if (is.null(colnames(x_val))) {
              colnames(x_val) <- seq_len(ncol(x_val))
            }
            colnames(x_val) <- paste(x, ".", colnames(x_val), sep = "")
          }
          x_val
        }
      )
      values_save <- do.call(cbind, values_save)
      row.names(values_save) <- seq_len(nrow(values_save))
    }

    values_save
  }

  clear <- function(plotter = FALSE) {
    env <<- new.env()
    env$.__n <- 1
    gc_last_time <<- gc.time()[3]
    values_save <<- list()
    last_time <<- bench::hires_time()
    plotter_expr_mod <- is.call(plotter) || is.expression(plotter)

    if (plotter_expr_mod) {
      plotter_expr <<- plotter_expr_mod
      plotter <<- plotter
      plotter <- FALSE
    }

    if (plotter_expr && isTRUE(plotter)) {
      new_plotter_window(...)
    }
  }

  structure(
    list(
      tracer = tracer,
      get = get,
      clear = clear
    ),
    class = "tracer"
  )
}

# If x is a matrix object which encodes a column vector, then the transpose of x
# is returned. Otherwise, x is returned.
col_to_row <- function(x) {
  if (inherits(x, "matrix") && nrow(x) > 1 && ncol(x) == 1) {
    t(x)
  } else {
    x
  }
}

# Setting up a new plot window
new_plotter_window <- function(...) {
  graphics::plot.new()
  graphics::plot.window(...)
  graphics::Axis(side = 1)
  graphics::Axis(side = 2)
}

#' Subsetting tracer objects
#'
#' @param x a tracer object.
#' @param i the indices of the trace information to extract.
#' @param j currently ignored.
#' @param ... other arguments passed on to \code{get}.
#' @param drop simplify a list with one element to a vector if TRUE.
#'
#' @return a list
#' @export
#'
#' @examples
#' tr <- tracer("i", Delta = 0)
#'
#' for (i in 1:3) {
#'   tr$tracer()
#' }
#'
#' tr[1]
'[.tracer' <- function(x, i, j, ..., drop = TRUE) {
  values <- x$get(...)[i]
  if (length(values) == 1 && drop) {
    values <- values[[1]]
  }
  values
}

#' Summarize and print trace information
#'
#' @param object   a tracer object
#' @param x        a tracer object
#' @param ...      other arguments (currently ignored)
#'
#' @return \code{summary} returns a data frame (of class trace) with columns
#' containing the values
#' of the traced objects, and if time is traced an additional column, \code{.time},
#' containing the cumulative runtime in seconds.
#' @export
#'
#' @examples
#' tr <- tracer("i", Delta = 0)
#'
#' for (i in 1:3) {
#'   tr$tracer()
#' }
#'
#' summary(tr)
#' print(tr)
summary.tracer <- function(object, ...) {
  object <- suppressWarnings(object$get(simplify = TRUE))
  if (".time" %in% colnames(object)) {
    object[, ".time"] <- c(0, cumsum(object[-1, ".time"]))
  }
  structure(as.data.frame(object), class = c("trace", "data.frame"))
}

#' @rdname summary.tracer
#' @export
print.tracer <- function(x, ...) print(x$get(simplify = TRUE))

#' Plot results from a trace
#'
#' Plots the value of a traced object on a log-scale against runtime.
#'
#' @param object a trace or tracer object
#' @param y      the name of the traced object to plot
#' @param log    logical. Should the y-axis be on a log-scale. Default is TRUE.
#' @param ...    additional arguments passed to
#'
#' @return a ggplot object
#' @import ggplot2
#' @export
#'
#' @examples
#' tr <- tracer("i", Delta = 0)
#'
#' for (i in 1:3) {
#'   tr$tracer()
#' }
#'
#' ggplot2::autoplot(tr, i)
autoplot.tracer <- function(object, y, ...) {
  autoplot(summary(object), {{ y }}, ...)
}

#' @rdname autoplot.tracer
#' @import ggplot2
#' @importFrom rlang .data
#' @export
autoplot.trace <- function(object, y, log = TRUE, ...) {
  p <- ggplot(object, aes(x = .data$.time, {{ y }})) +
    geom_point() +
    xlab("time (s)")
  if (log) {
    p <- p + scale_y_log10()
  }
  p
}

#' Constructor of plotter expression
#'
#' The plotter function returns a quoted expression that adds points
#' to a current plot. For use with tracer and terminator objects to
#' iteratively update plots during long running function evaluations.
#'
#' @param y a name of a symbol to plot
#' @param col point and line color
#' @param lty line type
#' @param pch plot symbol
#'
#' @return a quoted expression
#' @export
#'
#' @examples
#' plotter("i")
plotter <- function(y, col = "black", lty = "solid", pch = 1) {
  bquote(
    {
      .__y_val <- get(.(y))
      points(.__n, .__y_val, col = .(col), pch = .(pch))

      if (exists(".__n_old") && exists(".__y_val_old")) {
        lines(
          c(.__n_old, .__n),
          c(.__y_val_old, .__y_val),
          col = .(col),
          lty = .(lty)
        )
      }

      .__n_old <- .__n
      .__y_val_old <- .__y_val
    }
  )
}

test_that("plotter creates an expression", {
  p <- plotter("x")
  expect_true(is.call(p) || is.expression(p))
})

test_that("plotter accepts color argument", {
  p <- plotter("x", col = "red")
  expect_true(is.call(p))
  # Check that the expression was created successfully
  expect_true(length(p) > 0)
})

test_that("plotter accepts line type argument", {
  p <- plotter("x", lty = "dashed")
  expect_true(is.call(p))
  # Check that the expression was created successfully
  expect_true(length(p) > 0)
})

test_that("plotter accepts point character argument", {
  p <- plotter("x", pch = 16)
  expect_true(is.call(p))
  # Check that the expression was created successfully
  expect_true(length(p) > 0)
})

test_that("tracer object is created correctly", {
  tr <- tracer(c("x", "y"))
  expect_s3_class(tr, "tracer")
  expect_true(is.list(tr))
  expect_true(is.function(tr$tracer))
  expect_true(is.function(tr$get))
  expect_true(is.function(tr$clear))
})

test_that("tracer collects values correctly", {
  tr <- tracer(c("x", "y"), Delta = 0, save = TRUE, time = FALSE)

  test_fun <- function(n, cb) {
    for (i in 1:n) {
      x <- i
      y <- i^2
      cb()
    }
  }

  test_fun(5, tr$tracer)

  values <- tr$get()
  expect_length(values, 5)
  expect_equal(values[[1]]$x, 1)
  expect_equal(values[[1]]$y, 1)
  expect_equal(values[[5]]$x, 5)
  expect_equal(values[[5]]$y, 25)
})

test_that("tracer with expression works", {
  tr <- tracer(
    c("m", "m_sq"),
    expr = quote(m_sq <- m^2),
    Delta = 0,
    time = FALSE
  )

  test_fun <- function(n, cb) {
    for (i in 1:n) {
      m <- 2 * i
      cb()
    }
  }

  test_fun(3, tr$tracer)

  values <- tr$get()
  expect_length(values, 3)
  expect_equal(values[[1]]$m, 2)
  expect_equal(values[[1]]$m_sq, 4)
  expect_equal(values[[3]]$m, 6)
  expect_equal(values[[3]]$m_sq, 36)
})

test_that("tracer clear works", {
  tr <- tracer(c("x"), Delta = 0, time = FALSE)

  test_fun <- function(n, cb) {
    for (i in 1:n) {
      x <- i
      cb()
    }
  }

  test_fun(3, tr$tracer)
  expect_length(tr$get(), 3)

  tr$clear()
  expect_length(tr$get(), 0)
})

test_that("tracer subsetting works", {
  tr <- tracer(c("x"), Delta = 0, time = FALSE)

  test_fun <- function(n, cb) {
    for (i in 1:n) {
      x <- i
      cb()
    }
  }

  test_fun(5, tr$tracer)

  expect_equal(tr[1]$x, 1)
  expect_equal(tr[5]$x, 5)
  expect_length(tr[1:3], 3)
})

test_that("summary.tracer works", {
  tr <- tracer(c("x"), Delta = 0, time = TRUE)

  test_fun <- function(n, cb) {
    for (i in 1:n) {
      x <- i
      cb()
    }
  }

  test_fun(3, tr$tracer)

  s <- summary(tr)
  expect_s3_class(s, "trace")
  expect_s3_class(s, "data.frame")
  expect_equal(nrow(s), 3)
  expect_true(".time" %in% colnames(s))
})

test_that("tracer with time tracking works", {
  tr <- tracer(c("x"), Delta = 0, save = TRUE, time = TRUE)

  test_fun <- function(n, cb) {
    for (i in 1:n) {
      x <- i
      Sys.sleep(0.01)
      cb()
    }
  }

  test_fun(2, tr$tracer)

  values <- tr$get()
  expect_true(".time" %in% names(values[[1]]))
  expect_true(values[[1]]$.time >= 0)
})

#' Random number stream based on caching
#'
#' A random number stream uses a vectorized random number generator to generate
#' a cache of random numbers that can then be used sequentially. Whenever the
#' cache runs empty new numbers are generated automatically.
#'
#' @param m initial cache size
#' @param rng a random number generator
#' @param ... additional arguments passed to the random number generator
#'
#' @return A function that extracts random numbers from the cache and
#' fills the cache whenever it runs empty.
#' @export
#'
#' @examples
#' runif_stream <- rng_stream(10, runif, min = -1, max = 1)
#' runif_stream()
rng_stream <- function(m, rng, ...) {
  args <- list(...)
  cache <- do.call(rng, c(m, args))
  j <- 0
  fact <- 1

  next_rn <- function(r = m) {
    j <<- j + 1
    if (j > m) {
      if (fact == 1 && r < m) {
        fact <<- m / (m - r)
      }
      m <<- floor(fact * (r + 1))
      cache <<- do.call(rng, c(m, args))
      j <<- 1
    }
    cache[j]
  }

  next_rn
}

test_that("rng_stream creates a function", {
  stream <- rng_stream(10, runif)
  expect_true(is.function(stream))
})

test_that("rng_stream generates random numbers", {
  stream <- rng_stream(10, runif)
  val1 <- stream()
  val2 <- stream()
  expect_true(is.numeric(val1))
  expect_true(is.numeric(val2))
  expect_length(val1, 1)
  expect_length(val2, 1)
})

test_that("rng_stream passes arguments correctly", {
  stream <- rng_stream(10, runif, min = -1, max = 1)
  vals <- replicate(20, stream())
  expect_true(all(vals >= -1))
  expect_true(all(vals <= 1))
})

test_that("rng_stream refills cache correctly", {
  stream <- rng_stream(5, runif)
  # Generate more than cache size
  vals <- replicate(10, stream())
  expect_length(vals, 10)
  expect_true(all(vals >= 0))
  expect_true(all(vals <= 1))
})

test_that("rng_stream works with different RNGs", {
  stream_norm <- rng_stream(10, rnorm, mean = 5, sd = 2)
  vals <- replicate(20, stream_norm())
  expect_true(is.numeric(vals))
  expect_length(vals, 20)
})

#' Constructor of a terminator object
#'
#' Terminator objects are used to write termination conditions
#' that can be tested via a callback function within another function
#' during its evaluation.
#'
#' Terminator objects are similar to tracer objects but serve a different
#' purpose. Like tracer objects, they can be used to trace, print and
#' plot values of a variable within the evaluation environment of another
#' function during its evaluation. The primary purpose of a terminator
#' object is, however, to evaluate a termination condition, which can
#' trigger termination of a loop. Terminator objects do not save trace
#' information and do not trace runtime.
#'
#' @param cond  a termination condition. Either an expression or call that
#'              evaluates to a logical.
#' @param Delta an integer specifying how often the termination condition
#'              is evaluated. \code{Delta = 0} means never, and otherwise
#'              the condition is evaluated every \code{Delta}-th iteration.
#'              \code{Delta = 1} is the default.
#' @param print a variable name to print or a logical. If FALSE, nothing is
#'              printed. If a variable name is given, that variable (if it exists)
#'              will be printed every \code{Delta}-th iteration. If TRUE (the
#'              default) the last variable in \code{cond} is printed
#'              every \code{Delta}-th iteration.
#' @param plotter an expression, possibly created by the \code{plotter} function.
#' @param ...     other arguments passed to \code{format} for printing and
#'                \code{plot.window}
#'
#' @return A terminator object containing the functions \code{terminator} and
#' \code{clear}.
#' @export
#'
#' @examples
#' term <- terminator(quote(i >= 3), print = FALSE)
#'
#' for (i in 1:5) {
#'   if (term$terminator()) {
#'     break
#'   }
#' }
#'
#' i
terminator <- function(
  cond = FALSE,
  Delta = 1,
  print = TRUE,
  plotter = NULL,
  ...
) {
  if (isTRUE(print)) {
    var <- all.names(cond, functions = FALSE)
    var <- var[length(var)]
  } else {
    if (is.character(print)) {
      var <- print
      print <- TRUE
    } else {
      print <- FALSE
    }
  }

  cond_expr <- is.call(cond) || is.expression(cond)
  plotter_expr <- is.call(plotter) || is.expression(plotter)

  # Environment for evaluation of the condition
  env <- new.env()
  env$.__n <- 1

  if (plotter_expr) {
    new_plotter_window(...)
  }

  terminator <- function() {
    terminate <- isTRUE(cond)
    n <- env$.__n

    if (isTRUE(Delta > 0) && n %% Delta == 0) {
      # The condition is evaluated in its own environment to ensure that it
      # does not accidentally overwrite variables in the calling environment.
      # To give the expression access to the calling environment it is assigned
      # as the enclosing environment.
      parent.env(env) <- parent.frame()
      if (cond_expr) {
        terminate <- tryCatch(eval(cond, envir = env), error = function(e) {
          warning(e)
        })
      }
      terminate <- isTRUE(terminate)
      if (plotter_expr) {
        tryCatch(eval(plotter, envir = env), error = function(e) warning(e))
      }

      if (print) {
        values <- mget(var, envir = env, ifnotfound = NA)
        cat(
          paste(
            "n = ",
            n,
            ": ",
            var,
            " = ",
            format(values, ...),
            "; ",
            sep = ""
          ),
          "terminate: ",
          terminate,
          "\n",
          sep = ""
        )
      }
    }
    env$.__n <- n + 1
    invisible(terminate)
  }

  clear <- function(plotter = FALSE) {
    env <<- new.env()
    env$.__n <- 1
    plotter_expr_mod <- is.call(plotter) || is.expression(plotter)

    if (plotter_expr_mod) {
      plotter_expr <<- plotter_expr_mod
      plotter <<- plotter
      plotter <- FALSE
    }

    if (plotter_expr && isTRUE(plotter)) {
      new_plotter_window(...)
    }
  }

  structure(list(terminator = terminator, clear = clear), class = "terminator")
}

#' Print terminator information
#'
#' @param x   a terminator object
#' @param ... other arguments (currently ignored)
#'
#' @return The list of objects from the terminator's evaluation environment,
#' returned invisibly.
#'
#' @export
#'
#' @examples
#' term <- terminator(quote(i >= 3), print = FALSE)
#'
#' for (i in 1:5) {
#'   if (term$terminator()) {
#'     break
#'   }
#' }
#'
#' print(term, all.names = TRUE)
print.terminator <- function(x, ...) {
  env <- environment(x$terminator)$env
  values <- mget(ls(env, ...), env)

  print(values)
}

test_that("terminator object is created correctly", {
  term <- terminator()
  expect_s3_class(term, "terminator")
  expect_true(is.list(term))
  expect_true(is.function(term$terminator))
  expect_true(is.function(term$clear))
})

test_that("terminator with simple condition works", {
  term <- terminator(cond = FALSE, Delta = 1, print = FALSE)
  expect_false(term$terminator())
})

test_that("terminator with expression condition works", {
  term <- terminator(cond = quote(x > 5), Delta = 1, print = FALSE)

  test_fun <- function(cb) {
    for (i in 1:10) {
      x <- i
      if (cb()) break
    }
    x
  }

  result <- test_fun(term$terminator)
  expect_equal(result, 6) # Should stop when x > 5
})

test_that("terminator clear works", {
  term <- terminator(cond = quote(x > 3), Delta = 1, print = FALSE)

  test_fun <- function(cb) {
    for (i in 1:10) {
      x <- i
      if (cb()) break
    }
    x
  }

  result1 <- test_fun(term$terminator)
  expect_equal(result1, 4)

  term$clear()
  result2 <- test_fun(term$terminator)
  expect_equal(result2, 4) # Should work the same after clear
})

test_that("terminator with Delta works", {
  counter <- 0
  term <- terminator(cond = FALSE, Delta = 2, print = FALSE)

  for (i in 1:6) {
    term$terminator()
    counter <- counter + 1
  }

  expect_equal(counter, 6) # Should complete all iterations
})

test_that("print.terminator works", {
  term <- terminator(cond = FALSE, Delta = 1, print = FALSE)
  term$terminator()
  # Just check that print doesn't error
  expect_no_error(print(term))
})

#' Force evaluation of all arguments to a function
#'
#' Forces the evaluation of all arguments in the calling
#' environment of this function.
#'
#' Use `force_all()` as syntactic sugar to force evaluation of
#' all arguments to a function and thereby circumvent lazy evaluation.
#' If called from within a function with `...` as formal argument,
#' use `force_all(...)` to force evaluation of arguments captured
#' by `...`.
#'
#' @param ... arguments captured by ... are also evaluated if passed via ...
#'
#' @return NULL invisibly
#' @seealso [force()]
#' @export
#'
#' @examples
#' affine <- function(a, b) {
#'   function(x) a * x + b
#' }
#'
#' a <- 1
#' b <- 1
#'
#' affine_11 <- affine(a, b)
#'
#' a <- 2
#' b <- 2
#'
#' affine_11(1) # Gives 4 and not 2 due to lazy evaluation
#'
#' affine_forced <- function(a, b) {
#'   force_all()
#'   function(x) a * x + b
#' }
#'
#' a <- 1
#' b <- 1
#'
#' affine_11 <- affine_forced(a, b)
#'
#' a <- 2
#' b <- 2
#'
#' affine_11(1) # Gives 2
force_all <- function(...) {
  as.list(parent.frame())
  list(...)
  invisible(NULL)
}

test_that("force_all forces evaluation", {
  affine_lazy <- function(a, b) {
    function(x) a * x + b
  }

  affine_forced <- function(a, b) {
    force_all()
    function(x) a * x + b
  }

  # Test lazy evaluation (without force_all)
  a <- 1
  b <- 1
  f_lazy <- affine_lazy(a, b)
  a <- 2
  b <- 2
  expect_equal(f_lazy(1), 4) # Uses new values due to lazy evaluation

  # Test forced evaluation (with force_all)
  a <- 1
  b <- 1
  f_forced <- affine_forced(a, b)
  a <- 2
  b <- 2
  expect_equal(f_forced(1), 2) # Uses original values
})

test_that("force_all returns NULL invisibly", {
  result <- withVisible(force_all())
  expect_null(result$value)
  expect_false(result$visible)
})

test_that("force_all works with ... arguments", {
  test_fun <- function(a, b, ...) {
    force_all(...)
    list(a, b, list(...))
  }

  result <- test_fun(1, 2, x = 3, y = 4)
  expect_equal(result[[1]], 1)
  expect_equal(result[[2]], 2)
  expect_named(result[[3]], c("x", "y"))
})

#' Nuuk temperature data
#'
#' Annual summaries of monthly mean temperatures in Nuuk from 1867 to 2013.
#'
#' @format
#' A data frame with 147 rows and 6 columns:
#' \describe{
#'   \item{Year}{Year (numeric)}
#'   \item{Temperature}{Annual mean temperature}
#'   \item{Median}{Annual median of monthly mean temperatures}
#'   \item{High}{Annual maximum of monthly mean temperatures}
#'   \item{Low}{Annual minimum of monthly mean temperatures}
#'   \item{Range}{High - Low}
#' }
#'
#' @source <https://crudata.uea.ac.uk/cru/data/greenland/>
"nuuk"

#' South west Greenland temperature data
#'
#' Monthly average temperatures in degree Celsius in Nuuk and Qaqortoq from 1873 to 2013.
#'
#' @format
#' A data frame with 1692 rows and 5 columns:
#' \describe{
#'   \item{Year}{Year (numeric)}
#'   \item{Month}{Month (integer, 1 to 12)}
#'   \item{Temp_nuuk}{Monthly mean temperature in Nuuk}
#'   \item{Temp_Qaqortoq}{Monthly mean temperature in Qaqortoq}
#'   \item{Temp_diff}{Temperature difference, `Temp_nuuk - Temp_Qaqortoq`}
#' }
#'
#' @source <https://crudata.uea.ac.uk/cru/data/greenland/>
"greenland"

#' Social media news sharing
#'
#' A dataset containing features about articles published by Mashable in a period
#' of two years from January 7, 2013, to January 7, 2015. The purpose of collecting
#' the data was to predict the number of shares of the news articles on social networks.
#' Compared to the source, seven features are excluded. Two (url and a timestamp)
#' are not relevant predictors and five are redundant or almost redundant, leading
#' to collinearity.
#'
#' @format
#' A data frame with 39,644 rows and 54 columns:
#' \describe{
#'   \item{n_tokens_title}{Number of words in the title}
#'   \item{n_tokens_content}{Number of words in the content}
#'   \item{n_unique_tokens}{Rate of unique words in the content}
#'   \item{num_hrefs}{Number of links}
#'   \item{num_self_hrefs}{Number of links to other articles published by Mashable}
#'   \item{num_imgs}{Number of images}
#'   \item{num_videos}{Number of videos}
#'   \item{average_token_length}{Average length of the words in the content}
#'   \item{num_keywords}{Number of keywords in the metadata}
#'   \item{data_channel_is_lifestyle}{Is data channel 'Lifestyle'?}
#'   \item{data_channel_is_entertainment}{Is data channel 'Entertainment'?}
#'   \item{data_channel_is_bus}{Is data channel 'Business'?}
#'   \item{data_channel_is_socmed}{Is data channel 'Social Media'?}
#'   \item{data_channel_is_tech}{Is data channel 'Tech'?}
#'   \item{data_channel_is_world}{Is data channel 'World'?}
#'   \item{kw_min_min}{Worst keyword (min. shares)}
#'   \item{kw_max_min}{Worst keyword (max. shares)}
#'   \item{kw_avg_min}{Worst keyword (avg. shares)}
#'   \item{kw_max_max}{Best keyword (max. shares)}
#'   \item{kw_avg_max}{Best keyword (avg. shares)}
#'   \item{kw_min_avg}{Avg. keyword (min. shares)}
#'   \item{kw_max_avg}{Avg. keyword (max. shares)}
#'   \item{kw_avg_avg}{Avg. keyword (avg. shares)}
#'   \item{self_reference_min_shares}{Min. shares of referenced articles in Mashable}
#'   \item{self_reference_avg_sharess}{Avg. shares of referenced articles in Mashable}
#'   \item{weekday_is_monday}{Was the article published on a Monday?}
#'   \item{weekday_is_tuesday}{Was the article published on a Tuesday?}
#'   \item{weekday_is_wednesday}{Was the article published on a Wednesday?}
#'   \item{weekday_is_thursday}{Was the article published on a Thursday?}
#'   \item{weekday_is_friday}{Was the article published on a Friday?}
#'   \item{weekday_is_saturday}{Was the article published on a Saturday?}
#'   \item{weekday_is_sunday}{Was the article published on a Sunday?}
#'   \item{LDA_00}{Closeness to LDA topic 0}
#'   \item{LDA_01}{Closeness to LDA topic 1}
#'   \item{LDA_02}{Closeness to LDA topic 2}
#'   \item{LDA_03}{Closeness to LDA topic 3}
#'   \item{LDA_04}{Closeness to LDA topic 4}
#'   \item{global_subjectivity}{Text subjectivity}
#'   \item{global_sentiment_polarity}{Text sentiment polarity}
#'   \item{global_rate_positive_words}{Rate of positive words in the content}
#'   \item{global_rate_negative_words}{Rate of negative words in the content}
#'   \item{rate_positive_words}{Rate of positive words among non-neutral tokens}
#'   \item{rate_negative_words}{Rate of negative words among non-neutral tokens}
#'   \item{avg_positive_polarity}{Avg. polarity of positive words}
#'   \item{min_positive_polarity}{Min. polarity of positive words}
#'   \item{max_positive_polarity}{Max. polarity of positive words}
#'   \item{avg_negative_polarity}{Avg. polarity of negative  words}
#'   \item{min_negative_polarity}{Min. polarity of negative  words}
#'   \item{max_negative_polarity}{Max. polarity of negative  words}
#'   \item{title_subjectivity}{Title subjectivity}
#'   \item{title_sentiment_polarity}{Title polarity}
#'   \item{abs_title_subjectivity}{Absolute subjectivity level}
#'   \item{abs_title_sentiment_polarity}{Absolute polarity level}
#'   \item{shares}{Number of shares (target)}
#' }
#'
#' @source <https://archive.ics.uci.edu/dataset/332/online+news+popularity>
"news"

#' Vegetable sale
#'
#' Weekly sale of frozen vegetables in different stores.
#'
#' @format
#' A data frame with 1066 rows and 3 columns:
#' \describe{
#'   \item{\code{sale}}{a numeric vector. Number of items sold in a week}
#'   \item{\code{normalSale}}{a numeric vector. Estimated normal sale that week}
#'   \item{\code{store}}{a character vector. Id of the different stores}
#' }
#'
#' @source Obtained by package author.
"vegetables"

#' Torsion angles
#'
#' The phi and psi torsion angles between peptide planes for the human protein
#' 1HMP.
#'
#' @format
#' A data frame with 419 rows and 5 columns:
#' \describe{
#'   \item{\code{chain}}{a character. The chain (either A or B) of the protein}
#'   \item{\code{AA}}{a character. The three-letter amino acid name}
#'   \item{\code{pos}}{an integer. Position in the amino acid sequence}
#'   \item{\code{phi}}{numeric. The phi angle}
#'   \item{\code{psi}}{numeric. The psi angle}
#' }
#'
#' @source <https://warwick.ac.uk/fac/sci/moac/people/students/peter_cock/r/ramachandran>
"angle"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tracer.R
\name{plotter}
\alias{plotter}
\title{Constructor of plotter expression}
\usage{
plotter(y, col = "black", lty = "solid", pch = 1)
}
\arguments{
\item{y}{a name of a symbol to plot}

\item{col}{point and line color}

\item{lty}{line type}

\item{pch}{plot symbol}
}
\value{
a quoted expression
}
\description{
The plotter function returns a quoted expression that adds points
to a current plot. For use with tracer and terminator objects to
iteratively update plots during long running function evaluations.
}
\examples{
plotter("i")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/terminator.R
\name{print.terminator}
\alias{print.terminator}
\title{Print terminator information}
\usage{
\method{print}{terminator}(x, ...)
}
\arguments{
\item{x}{a terminator object}

\item{...}{other arguments (currently ignored)}
}
\value{
The list of objects from the terminator's evaluation environment,
returned invisibly.
}
\description{
Print terminator information
}
\examples{
term <- terminator(quote(i >= 3), print = FALSE)

for (i in 1:5) {
  if (term$terminator()) {
    break
  }
}

print(term, all.names = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tracer.R
\name{summary.tracer}
\alias{summary.tracer}
\alias{print.tracer}
\title{Summarize and print trace information}
\usage{
\method{summary}{tracer}(object, ...)

\method{print}{tracer}(x, ...)
}
\arguments{
\item{object}{a tracer object}

\item{...}{other arguments (currently ignored)}

\item{x}{a tracer object}
}
\value{
\code{summary} returns a data frame (of class trace) with columns
containing the values
of the traced objects, and if time is traced an additional column, \code{.time},
containing the cumulative runtime in seconds.
}
\description{
Summarize and print trace information
}
\examples{
tr <- tracer("i", Delta = 0)

for (i in 1:3) {
  tr$tracer()
}

summary(tr)
print(tr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{angle}
\alias{angle}
\title{Torsion angles}
\format{
A data frame with 419 rows and 5 columns:
\describe{
\item{\code{chain}}{a character. The chain (either A or B) of the protein}
\item{\code{AA}}{a character. The three-letter amino acid name}
\item{\code{pos}}{an integer. Position in the amino acid sequence}
\item{\code{phi}}{numeric. The phi angle}
\item{\code{psi}}{numeric. The psi angle}
}
}
\source{
\url{https://warwick.ac.uk/fac/sci/moac/people/students/peter_cock/r/ramachandran}
}
\usage{
angle
}
\description{
The phi and psi torsion angles between peptide planes for the human protein
1HMP.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tracer.R
\name{tracer}
\alias{tracer}
\title{Constructor of a tracer object}
\usage{
tracer(
  objects = NULL,
  Delta = 1,
  save = TRUE,
  time = TRUE,
  expr = NULL,
  plotter = NULL,
  ...
)
}
\arguments{
\item{objects}{a character vector of names of the objects that are to be traced.
The objects are searched for in the calling environment of the
\code{tracer} function. Objects created by the \code{expr} argument
can also be traced.}

\item{Delta}{an integer specifying if and how often trace information is printed.
\code{Delta = 0} means never, and otherwise trace information is printed
every \code{Delta}-th iteration. \code{Delta = 1} is the default.}

\item{save}{a logical value. Determines if the trace information is stored.}

\item{time}{a logical value. Determines if runtime information in seconds
is traced.}

\item{expr}{an expression that will be evaluated in an environment that has
the calling environment of the \code{tracer} function as parent.}

\item{plotter}{an expression, possibly created by the \code{plotter} function.}

\item{...}{other arguments passed to \code{format} for printing and
\code{plot.window}}
}
\value{
A tracer object containing the functions \code{tracer}, \code{get} and
\code{clear}.
}
\description{
Tracer objects can collect, print and summarize trace information from
the evaluation environment of other functions during their evaluation.
}
\details{
The function \code{tracer} constructs a tracer object containing a
\code{tracer}, a \code{get} and a \code{clear} function. A call of
the objects \code{tracer} function can
be inserted in other functions and used to collect, print and plot trace information
about the internals of that function during its evaluation. The objects \code{get}
function can access that information afterwards, and its \code{clear} function deletes
all stored values in the tracer object.

A tracer object can trace time (in seconds) between \code{tracer} calls,
which are measured by
the \code{hires_time} function from the bench package. There are \code{print}
and \code{summary} methods available for summarizing the trace information.
A call of the \code{tracer} function can be manually inserted into the body of
the function that is to be traced, it can be inserted using \code{base::trace},
or it can be passed as an argument to any function with a callback argument.
}
\examples{
test_tracer <- tracer(c("m", "m_sq"), expr = quote(m_sq <- m^2))

test <- function(n, cb = NULL) {
  for(i in 1:n) {
    m <- 2 * i
    Sys.sleep(0.1)
    if (!is.null(cb)) cb()
  }
}

test(10, test_tracer$tracer)
summary(test_tracer)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/terminator.R
\name{terminator}
\alias{terminator}
\title{Constructor of a terminator object}
\usage{
terminator(cond = FALSE, Delta = 1, print = TRUE, plotter = NULL, ...)
}
\arguments{
\item{cond}{a termination condition. Either an expression or call that
evaluates to a logical.}

\item{Delta}{an integer specifying how often the termination condition
is evaluated. \code{Delta = 0} means never, and otherwise
the condition is evaluated every \code{Delta}-th iteration.
\code{Delta = 1} is the default.}

\item{print}{a variable name to print or a logical. If FALSE, nothing is
printed. If a variable name is given, that variable (if it exists)
will be printed every \code{Delta}-th iteration. If TRUE (the
default) the last variable in \code{cond} is printed
every \code{Delta}-th iteration.}

\item{plotter}{an expression, possibly created by the \code{plotter} function.}

\item{...}{other arguments passed to \code{format} for printing and
\code{plot.window}}
}
\value{
A terminator object containing the functions \code{terminator} and
\code{clear}.
}
\description{
Terminator objects are used to write termination conditions
that can be tested via a callback function within another function
during its evaluation.
}
\details{
Terminator objects are similar to tracer objects but serve a different
purpose. Like tracer objects, they can be used to trace, print and
plot values of a variable within the evaluation environment of another
function during its evaluation. The primary purpose of a terminator
object is, however, to evaluate a termination condition, which can
trigger termination of a loop. Terminator objects do not save trace
information and do not trace runtime.
}
\examples{
term <- terminator(quote(i >= 3), print = FALSE)

for (i in 1:5) {
  if (term$terminator()) {
    break
  }
}

i
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rng.R
\name{rng_stream}
\alias{rng_stream}
\title{Random number stream based on caching}
\usage{
rng_stream(m, rng, ...)
}
\arguments{
\item{m}{initial cache size}

\item{rng}{a random number generator}

\item{...}{additional arguments passed to the random number generator}
}
\value{
A function that extracts random numbers from the cache and
fills the cache whenever it runs empty.
}
\description{
A random number stream uses a vectorized random number generator to generate
a cache of random numbers that can then be used sequentially. Whenever the
cache runs empty new numbers are generated automatically.
}
\examples{
runif_stream <- rng_stream(10, runif, min = -1, max = 1)
runif_stream()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{vegetables}
\alias{vegetables}
\title{Vegetable sale}
\format{
A data frame with 1066 rows and 3 columns:
\describe{
\item{\code{sale}}{a numeric vector. Number of items sold in a week}
\item{\code{normalSale}}{a numeric vector. Estimated normal sale that week}
\item{\code{store}}{a character vector. Id of the different stores}
}
}
\source{
Obtained by package author.
}
\usage{
vegetables
}
\description{
Weekly sale of frozen vegetables in different stores.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tracer.R
\name{[.tracer}
\alias{[.tracer}
\title{Subsetting tracer objects}
\usage{
\method{[}{tracer}(x, i, j, ..., drop = TRUE)
}
\arguments{
\item{x}{a tracer object.}

\item{i}{the indices of the trace information to extract.}

\item{j}{currently ignored.}

\item{...}{other arguments passed on to \code{get}.}

\item{drop}{simplify a list with one element to a vector if TRUE.}
}
\value{
a list
}
\description{
Subsetting tracer objects
}
\examples{
tr <- tracer("i", Delta = 0)

for (i in 1:3) {
  tr$tracer()
}

tr[1]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tracer.R
\name{autoplot.tracer}
\alias{autoplot.tracer}
\alias{autoplot.trace}
\title{Plot results from a trace}
\usage{
\method{autoplot}{tracer}(object, y, ...)

\method{autoplot}{trace}(object, y, log = TRUE, ...)
}
\arguments{
\item{object}{a trace or tracer object}

\item{y}{the name of the traced object to plot}

\item{...}{additional arguments passed to}

\item{log}{logical. Should the y-axis be on a log-scale. Default is TRUE.}
}
\value{
a ggplot object
}
\description{
Plots the value of a traced object on a log-scale against runtime.
}
\examples{
tr <- tracer("i", Delta = 0)

for (i in 1:3) {
  tr$tracer()
}

ggplot2::autoplot(tr, i)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/force.R
\name{force_all}
\alias{force_all}
\title{Force evaluation of all arguments to a function}
\usage{
force_all(...)
}
\arguments{
\item{...}{arguments captured by ... are also evaluated if passed via ...}
}
\value{
NULL invisibly
}
\description{
Forces the evaluation of all arguments in the calling
environment of this function.
}
\details{
Use \code{force_all()} as syntactic sugar to force evaluation of
all arguments to a function and thereby circumvent lazy evaluation.
If called from within a function with \code{...} as formal argument,
use \code{force_all(...)} to force evaluation of arguments captured
by \code{...}.
}
\examples{
affine <- function(a, b) {
  function(x) a * x + b
}

a <- 1
b <- 1

affine_11 <- affine(a, b)

a <- 2
b <- 2

affine_11(1) # Gives 4 and not 2 due to lazy evaluation

affine_forced <- function(a, b) {
  force_all()
  function(x) a * x + b
}

a <- 1
b <- 1

affine_11 <- affine_forced(a, b)

a <- 2
b <- 2

affine_11(1) # Gives 2
}
\seealso{
\code{\link[=force]{force()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CSwR-package.R
\docType{package}
\name{CSwR-package}
\alias{CSwR}
\alias{CSwR-package}
\title{CSwR: Companion to the Book "Computational Statistics with R"}
\description{
Provides data sets and functions used in the book "Computational Statistics with R" (\url{https://cswr.nrhstat.org}).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://jolars.github.io/CSwR-Package/}
  \item \url{https://cswr.nrhstat.org/}
  \item Report bugs at \url{https://github.com/jolars/CSwR-Package/issues}
}

}
\author{
\strong{Maintainer}: Johan Larsson \email{johan@jolars.co} (\href{https://orcid.org/0000-0002-4029-5945}{ORCID})

Authors:
\itemize{
  \item Niels Richard Hansen \email{Niels.R.Hansen@math.ku.dk} (\href{https://orcid.org/0000-0003-3883-365X}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{greenland}
\alias{greenland}
\title{South west Greenland temperature data}
\format{
A data frame with 1692 rows and 5 columns:
\describe{
\item{Year}{Year (numeric)}
\item{Month}{Month (integer, 1 to 12)}
\item{Temp_nuuk}{Monthly mean temperature in Nuuk}
\item{Temp_Qaqortoq}{Monthly mean temperature in Qaqortoq}
\item{Temp_diff}{Temperature difference, \code{Temp_nuuk - Temp_Qaqortoq}}
}
}
\source{
\url{https://crudata.uea.ac.uk/cru/data/greenland/}
}
\usage{
greenland
}
\description{
Monthly average temperatures in degree Celsius in Nuuk and Qaqortoq from 1873 to 2013.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{nuuk}
\alias{nuuk}
\title{Nuuk temperature data}
\format{
A data frame with 147 rows and 6 columns:
\describe{
\item{Year}{Year (numeric)}
\item{Temperature}{Annual mean temperature}
\item{Median}{Annual median of monthly mean temperatures}
\item{High}{Annual maximum of monthly mean temperatures}
\item{Low}{Annual minimum of monthly mean temperatures}
\item{Range}{High - Low}
}
}
\source{
\url{https://crudata.uea.ac.uk/cru/data/greenland/}
}
\usage{
nuuk
}
\description{
Annual summaries of monthly mean temperatures in Nuuk from 1867 to 2013.
}
\keyword{datasets}
